@@ -213,7 +213,9 @@ def to_1D_vec(vec):
     np.array
         The original vector in a form of a 1D vector.
     '''
-    vec = np.squeeze(vec)
+    types = [int, float, np.float64, np.int64]
+    if not type(vec) in types and not len(vec) <= 1:
+        vec = np.squeeze(vec)
     if len(np.shape(vec)) >= 2:
         print('You have another not negligible axis')
     return vec
@@ -285,12 +287,12 @@ def find_peaks(x, y, to_plot= False, dist = None):
     Parameters
     ----------
     x : np.array, array_like
-        The Spacial coordinates vector.
+        The Spatial coordinates vector.
     y : np.array, array_like
-       The functional values of the spacial coordiantes vector. The definiion for `f(x) = y`. Usually will be the local potential vector.
-    to_plot : bool, optional, defualt: False
+       The functional values of the spatial coordiantes vector. The definiion for `f(x) = y`. Usually will be the local potential vector.
+    to_plot : bool, optional, default: False
         Flag that determines whether to enable plotting or not.
-    dist : float, optional, defualt: None
+    dist : float, optional, default: None
         The distance between peaks.
     Returns
     -------
@@ -340,12 +342,12 @@ def find_peaks_maxima(x, y, ignore_local_maxima = False, to_plot = False, promin
     Parameters
     ----------
     x : np.array, array_like
-        Spacial coordinates vector.
+        spatial coordinates vector.
     y : np.array, array_like
         Potential values v(z) vector.
-    ignore_local_maxima : bool, optional, defualt: False
+    ignore_local_maxima : bool, optional, default: False
         It enables to turn on the option to ignore local maxima points.
-    to_plot : bool, optional, defualt: False
+    to_plot : bool, optional, default: False
         Flag that determines whether to enable plotting or not.
     Returns
     -------
@@ -410,9 +412,21 @@ def find_peaks_maxima(x, y, ignore_local_maxima = False, to_plot = False, promin
                     if not np.abs(y[maxima_temp[0]][i] - y[maxima_temp[0]][i+1]) <= 0.12:
                         temp_flag = False
                 if temp_flag:
-                    promin_max = np.min(promin[0])
+                    try:
+                        promin_max = np.min(promin[0])
+                    except ValueError:
+                        if promin[0] is None or len(promin[0]) == 0:
+                            promin_max = 0.1
+                        else:
+                            raise  ValueError('Chceck Your input vectors')
                 else:
-                    promin_max = np.max(promin[0])-0.2
+                    try:
+                         promin_max = np.max(promin[0])-0.2
+                    except ValueError:
+                        if promin[0] is None or len(promin[0]) == 0:
+                            promin_max = 0.1
+                        else:
+                            raise  ValueError('Chceck Your input vectors')
             else:
                 promin_max = promin[0][peaks_1[0]]
             maxima = ss.find_peaks(y, prominence=np.min(promin_max)- 0.2 )
@@ -435,11 +449,12 @@ def get_prominence(z,v,num_of_min = None,is_max = False):
     Parameters
     ----------
     z : np.array, array_like
-        The spacial coordinates vector.
+        The spatial coordinates vector.
     v : np.array, array_like
        The potential values (v(z)) vector
-    num_of_min : int, optional, the default is None
+    num_of_min : int, optional, default : None
         Input to detemine how many kind of minima's we have in our system.
+    is_max : bool, optional, default : None
     Returns
     -------
     `int`
@@ -501,12 +516,12 @@ def find_peaks_minima(x, y, ignore_local_minima = False,to_plot = False, promine
     Parameters
     ----------
     x : np.array, array_like
-        The spacial coordinates vector.
+        The spatial coordinates vector.
     y : np.array, array_like
        The potential values (v(z)) vector
-    ignore_local_minima : bool, optional, defualt: False
+    ignore_local_minima : bool, optional, default: False
        It enables to turn on the option to ignore local minima points.
-    to_plot : bool, optional, defualt: False
+    to_plot : bool, optional, default: False
         This is a flag that determines whether to enable plotting or not.
     prominence : float, optional, the default is None
         Input to detemine the prominence threshold for the find_peak.
@@ -583,9 +598,21 @@ def find_peaks_minima(x, y, ignore_local_minima = False,to_plot = False, promine
                     if not np.abs(y[minima_temp[0]][i] - y[minima_temp[0]][i + 1]) <= 0.12:
                         temp_flag = False
                 if temp_flag:
-                    promin_max = np.min(promin[0])
+                    try:
+                        promin_max = np.min(promin[0])
+                    except ValueError:
+                        if promin[0] is None or len(promin[0]) == 0:
+                            promin_max = 0.1
+                        else:
+                            raise ValueError('Chceck Your input vectors')
                 else:
-                    promin_max = np.max(promin[0])-0.2
+                    try:
+                        promin_max = np.max(promin[0])-0.2
+                    except ValueError:
+                        if promin[0] is None or len(promin[0]) == 0:
+                            promin_max = 0.1
+                        else:
+                            raise ValueError('Chceck Your input vectors')
             else:
                 promin_max = promin[0][peaks_1[0]] - 0.2
 
@@ -631,7 +658,7 @@ def create_k_grid(N, L, dimention='1D'):
         The number of measuerment sample. The number of the k-grid points.
     L : int/float
          The actual length of the system. Should be given in units of [m].
-    dimention : str, {'1D', '2D'}, optional, defualt: '1D'
+    dimention : str, {'1D', '2D'}, optional, default: '1D'
         The choices are in the brackets. It defines the K-grid for 1D or 2D mesh.
     Returns
     -------
@@ -671,10 +698,10 @@ def normalize_wave_function_numerically(z,wave_function, units = 'Meter'):
     Parameters
     ----------
     z : np.array, array_like
-        The spacial coordinates vector where the wave function spreads.
+        The spatial coordinates vector where the wave function spreads.
     wave_function : np.array, array_like
         The complex values of the wave function in at the relevant z positions.
-    units : str, {'Meter', 'Angstrum'], optioanl, defualt: 'Meter'
+    units : str, {'Meter', 'Angstrum'], optioanl, default: 'Meter'
          The choices are in the brackets. It defines the units of the normilization factor.
     Returns
     -------
@@ -717,11 +744,11 @@ def normalize_wave_function(wave_equation, bound_up=np.inf, bound_down=0, units
         It is in a lambda function format. It can be a static function or a lambda state.
         The equation of the wave function we wish to normalize. In this project, mainly supplied as
         a gaussian wave function.
-    bound_up : int/float, optional, defualt: smp.oo
-        The intgration boundaries. (Defualt = inifinity)
-    bound_down : int/float, optional, defualt: -smp.oo
-        The intgration boundaries. (Defualt = -inifinity)
-    units : str, {'Meter', 'Angstrum'], optioanl, defualt: 'Meter'
+    bound_up : int/float, optional, default: smp.oo
+        The intgration boundaries. (default = inifinity)
+    bound_down : int/float, optional, default: -smp.oo
+        The intgration boundaries. (default = -inifinity)
+    units : str, {'Meter', 'Angstrum'], optioanl, default: 'Meter'
          The choices are in the brackets. It defines the units of the normilization factor.
     Returns
     -------
@@ -754,11 +781,11 @@ def normalize_wave_function_sym(wave_equation, bound_up=smp.oo, bound_down=0, un
         It is in a lambda function format. It can be a static function or a lambda state.
         The equation of the wave function we wish to normalize. In this project, mainly supplied as
         a gaussian wave function.
-    bound_up : int/float, optional, defualt: smp.oo
-        The intgration boundaries. (Defualt = inifinity)
-    bound_down : int/float, optional, defualt: -smp.oo
-        The intgration boundaries. (Defualt = -inifinity)
-    units : str, {'Meter', 'Angstrum'], optioanl, defualt: 'Meter'
+    bound_up : int/float, optional, default: smp.oo
+        The intgration boundaries. (default = inifinity)
+    bound_down : int/float, optional, default: -smp.oo
+        The intgration boundaries. (default = -inifinity)
+    units : str, {'Meter', 'Angstrum'], optioanl, default: 'Meter'
          The choices are in the brackets. It defines the units of the normilization factor.
     Returns
     -------
@@ -791,10 +818,10 @@ def is_normalized(z, psi_values,units = 'Meter'):
     Parameters
     ----------
     z : np.array, array_like
-        The spacial coordinates the wave function is specified at.
+        The spatial coordinates the wave function is specified at.
     psi_values : np.array, array_like
-        The wavefunction values along the spacial coordinates. Must suit the units for angstrums.
-    units : str, {'Meter', 'Angstrum'], optioanl, defualt: 'Meter'
+        The wavefunction values along the spatial coordinates. Must suit the units for angstrums.
+    units : str, {'Meter', 'Angstrum'], optioanl, default: 'Meter'
          The choices are in the brackets. It defines the units of the normilization factor.
     Returns
     -------
@@ -835,9 +862,9 @@ def gaussian_function_sym( k0, sigma, z0, dimention='1D', A=None):
         The standard devaiation of the gaussian wave-function.
     z0 : float
         Initial poisition where the wave function is centered at.
-    dimention : str, {'1D', '2D'}, optional, defualt: '1D'
+    dimention : str, {'1D', '2D'}, optional, default: '1D'
         The choices are in the brackets. It defines the K-grid for 1D or 2D mesh.
-    A : float, optional, defualt: None
+    A : float, optional, default: None
         If we already calculated the normaliztion factor we can supply it here.
         Otherwise it calculate from scratch here.
     Returns
@@ -857,16 +884,16 @@ def gaussian_function(x, k0, sigma, z0, dimention='1D', A=None):
     Parameters
     ----------
     x : np.array
-        The spacial coordinates vector.
+        The spatial coordinates vector.
     k0 : float
         The initial momentum value,
     sigma : float
         The standard devaiation of the gaussian wave-function.
     z0 : float
         Initial poisition where the wave function is centered at.
-    dimention : str, {'1D', '2D'}, optional, defualt: '1D'
+    dimention : str, {'1D', '2D'}, optional, default: '1D'
         The choices are in the brackets. It defines the K-grid for 1D or 2D mesh.
-    A : float, optional, defualt: None
+    A : float, optional, default: None
         If we already calculated the normaliztion factor we can supply it here.
         Otherwise it calculate from scratch here.
     Returns
@@ -890,16 +917,16 @@ def get_psi0(k0, sigma, z0, z, dimention='1D', sqrd_psi=False,units= 'Meter'):
         The standard devaiation of the gaussian wave-function.
     z0 : float
         Initial poisition where the wave function is centered at.
-    dimention : str, {'1D', '2D'}, optional, defualt: '1D'
+    dimention : str, {'1D', '2D'}, optional, default: '1D'
         The choices are in the brackets. It defines the K-grid for 1D or 2D mesh.
-    sqrd_psi : bool, optional, defualt: False
+    sqrd_psi : bool, optional, default: False
         It gives an optiona to return also the absolute sqrd of the wave function.
-    units : str, {'Meter', 'Angstrum'], optioanl, defualt: 'Meter'
+    units : str, {'Meter', 'Angstrum'], optioanl, default: 'Meter'
          The choices are in the brackets. It defines the units of the normilization factor.
     Returns
     -------
     np.array, np.array
-        The first vector represents the spacial coordinates grid
+        The first vector represents the spatial coordinates grid
         and the second vector represnts the wave function value at this position. the values are complex numbers.
     '''
     if dimention == '1D':
@@ -959,15 +986,15 @@ def get_first_derivative_fft(x, y, L=None):
     Parameters
     ----------
     x : np.array
-        The spacial coordinates vector.
+        The spatial coordinates vector.
     y : np.array
-        The appropriate function-values corresponds the spacial coordinates vector.
-    L : float, optional, defualt: None
+        The appropriate function-values corresponds the spatial coordinates vector.
+    L : float, optional, default: None
         It should be supplied with the length of the system. It stands for the creation of the K-grid vector.
     Returns
     -------
     np.array, np.array
-        It returns the spacial coordinates vector and the first derivative vecotr.
+        It returns the spatial coordinates vector and the first derivative vecotr.
     '''
     x = to_1D_vec(x)
     y = to_1D_vec(y)
@@ -990,15 +1017,15 @@ def get_second_derivative_fft(x, y, L=None):
     Parameters
     ----------
     x : np.array
-        The spacial coordinates vector.
+        The spatial coordinates vector.
     y : np.array
-        The appropriate function-values corresponds the spacial coordinates vector.
-    L : float, optional, defualt: None
+        The appropriate function-values corresponds the spatial coordinates vector.
+    L : float, optional, default: None
         It should be supplied with the length of the system. It stands for the creation of the K-grid vector.
     Returns
     -------
     np.array, np.array
-        It returns the spacial coordinates vector and the second derivative vecotr.
+        It returns the spatial coordinates vector and the second derivative vecotr.
     '''
     x = to_1D_vec(x)
     y = to_1D_vec(y)
@@ -1022,13 +1049,13 @@ def get_first_derivative(x, y):
     Parameters
     ----------
     x : np.array
-        The spacial coordinates vector.
+        The spatial coordinates vector.
     y : np.array
-        The appropriate function-values corresponds the spacial coordinates vector.
+        The appropriate function-values corresponds the spatial coordinates vector.
     Returns
     -------
     np.array, np.array
-        It returns the spacial coordinates vector and the first derivative vector.
+        It returns the spatial coordinates vector and the first derivative vector.
     '''
     x = to_1D_vec(x)
     y = to_1D_vec(y)
@@ -1042,13 +1069,13 @@ def get_second_derivative(x, y):
     Parameters
     ----------
     x : np.array
-        The spacial coordinates vector.
+        The spatial coordinates vector.
     y : np.array
-        The appropriate function-values corresponds the spacial coordinates vector.
+        The appropriate function-values corresponds the spatial coordinates vector.
     Returns
     -------
     np.array, np.array
-        It returns the spacial coordinates vector and the second derivative vector.
+        It returns the spatial coordinates vector and the second derivative vector.
     '''
     x = to_1D_vec(x)
     y = to_1D_vec(y)
@@ -1159,7 +1186,7 @@ def potential_expectation_nadav(psi0_values, dx, v):
     psi0_values : np.array
         The wave-function values in a vector.
     dx : float
-        The spacial grid spacing.
+        The spatial grid spacing.
     v : np.array
         The local potential vector.
     Returns
@@ -1180,8 +1207,8 @@ def kinetic_energy_operator(psi0_values, z, L=None):
     psi0_values : np.array
         The wave-function values in a vector.
     z : float
-        The spacial grid vector.
-    L : float, optional, defualt: None
+        The spatial grid vector.
+    L : float, optional, default: None
         The length of the system. if not suuplied it is calculated as
         the difference between the last value of z and its first value. L=z[-1]-z[0]
     Returns
@@ -1214,7 +1241,7 @@ def kinetic_energy_expectation_value(psi0_values, z):
     psi0_values : np.array
         The wave-function values in a vector.
     z : float
-        The spacial grid vector.
+        The spatial grid vector.
     Returns
     -------
     float
@@ -1233,7 +1260,7 @@ def initial_momentum_yair(sigma, E0, L, z0, A, T0):
     Parameters
     ----------
     sigma : float/int
-        Standard deviation in spacial grid for the gaussin wave function.
+        Standard deviation in spatial grid for the gaussin wave function.
     E0 : float/int
         Initial Energy. Usually stands for the energy value of an electorn located at the bootom of the
         conduction band.
@@ -1340,13 +1367,16 @@ def find_adjacent_minima_maxima(z,v,start_peak_pos,reverse = False,prominence=No
     Parameters
     ----------
     z : np.array
-        The spacial coordinates vector.
+        The spatial coordinates vector.
     v : np.array
         The local potential vector.
     start_peak_pos : float
         The initial position where to start seaching for the minima point.
-    reverse : bool, optional, defualt: None
+    reverse : bool, optional, default: None
         It enables to look for maxima to minima adjacent points. (The reverse order).
+    prominence : float, optional, default: None
+        The prominence of the peak. It helps to detemine what peaks we wish relate during the calculation.
+        For more inforamtion about this mathematical entity it is recommended to look for it on web (it has to be with topology in mathematics)
     Returns
     -------
     (int, float), (int, float)
@@ -1379,8 +1409,13 @@ def find_adjacent_minima_maxima(z,v,start_peak_pos,reverse = False,prominence=No
         if not reverse:
             return find_closest_value_in_array(minima_z_tot[minima_z_tot < maxima_z_global[-1]][-1],z) , find_closest_value_in_array(maxima_z_global[-1],z)
         else:
-            return find_closest_value_in_array(maxima_z_global[1], z) ,find_closest_value_in_array(minima_z_tot[minima_z_tot > maxima_z_global[1]][0],
-                                               z)
+            if len(maxima_z_global) == 1:
+                return find_closest_value_in_array(maxima_z_global[0], z), find_closest_value_in_array(
+                    minima_z_tot[minima_z_tot > maxima_z_global[0]][0],
+                    z)
+            else:
+                return find_closest_value_in_array(maxima_z_global[1], z) ,find_closest_value_in_array(minima_z_tot[minima_z_tot > maxima_z_global[1]][0],
+                                                   z)
     else:
 
         if not reverse:
@@ -1441,7 +1476,7 @@ def force_psi_units(psi, units = 'Meter'):
     ----------
     psi : np.array
         The vector of the wave-function values to be forced into the desired units.
-    units : str, {'Meter', 'Angstrum'], optioanl, defualt: 'Meter'
+    units : str, {'Meter', 'Angstrum'], optioanl, default: 'Meter'
          The choices are in the brackets. It defines the units of the normilization factor.
     Returns
     -------
@@ -1478,7 +1513,7 @@ def run_one_time_step(wave_function, time_step, v, z , k):
         This is the local potential vector.
     z : np.array.
         Should be as the same size as `wave_function` vector.
-        This is the spacial coordinates vector.
+        This is the spatial coordinates vector.
     k : np.array.
         Should be as the same size as `wave_function` vector.
         This the `k grid` axis vector.
@@ -1552,11 +1587,11 @@ def cumulative_probabilty(wave_function, z, z_cutoff, dz):
     wave_function : np.array
         The Wave function values vector to be propagated.
     z : np.array
-        The z spacial coordinates vector
+        The z spatial coordinates vector
     z_cutoff : float
         The maximum value of to calculate the accumulated probability.
     dz : float
-        The spacial gird differnce between points.
+        The spatial gird differnce between points.
     Returns
     -------
     float
@@ -1581,12 +1616,16 @@ def transmision_coeff(psi0,Nt,dt,v,z,k,z_position,path_to_save = None, final_tim
     v : np.array
         The local potential vector.
     z : np.array
-        The spacial coordinates vector.
+        The spatial coordinates vector.
     k : np.array
         K grid vector.
     z_position : float
         The position at which it calculates the transimission coefficient. Should
         be given in units of Meter.
+    path_to_save : string, optional, default:None
+        The absolute path to save the figure obtained from the calculation
+    final_time : float, optional, default:None
+        The top boundary for the integration as we can integrate until this given time of the simulation. It stands for **t_f**
     Returns
     -------
     float
@@ -1632,7 +1671,7 @@ def transmision_coeff(psi0,Nt,dt,v,z,k,z_position,path_to_save = None, final_tim
     ax.set_xlabel('Time simulation, in secs')
     ax.set_ylabel('Cumulative probabilty through the interface')
 
-    if final_time is None:
+    if final_time is None or final_time == 0:
         ax.plot(np.array(cons.sec2fs(dt * np.arange(0, Nt))), np.array(results[:,1]),
                   linewidth=2,
                   label=r'Probabilty current $J_{{z=z_{{interface}}}}$, Transmission coefficient is : {:.5f} $\frac{{e}}{{\AA}}$'.format(
@@ -1662,13 +1701,13 @@ def adjust_grid_density(density,z,v):
         The grid density. A quantity that was determined through the convergence tests and is the number
         of the space steps divided by the length of the system.
     z : np.array
-        The spacial coordinates vector.
+        The spatial coordinates vector.
     v : np.array
         The local potntial vector along the system.
     Returns
     -------
     np.array, np.array
-        It returns the new spacial coordinate vecor as z, and the new local potneital vector as v.
+        It returns the new spatial coordinate vecor as z, and the new local potneital vector as v.
         They have been adjusted to the defined grid density supplied.
     '''
 
@@ -1692,13 +1731,13 @@ def reach_plateau(x,y,critiria_plat = 0.0001, tol_2 = 0.005):
     Parameters
     ----------
     x : np.array
-        The independent variable, usually will be the spacial grid vector or time space vector.
+        The independent variable, usually will be the spatial grid vector or time space vector.
     y : np.array
         The dependent variable. The matching value of a function to the respective x value. Can be the comulative probabilty.
-    critiria_plat : float. optional, defualt: 0.0001
+    critiria_plat : float. optional, default: 0.0001
         The value we decide to relate as no change in slope to determine the plateau region. The closer to zero
         the stricter critiria.
-    tol_2 :  float. optional, defualt: 0.005
+    tol_2 :  float. optional, default: 0.005
         The value we decide to relate as no change in values of the cumulative probabilty. The smaller value -
         the stricter critiria.
     Returns
@@ -1748,14 +1787,14 @@ def find_interface(x, y):
     Parameters
     ----------
     x : np.array
-        It stands for the spacial coordinates array.
+        It stands for the spatial coordinates array.
     y : np.array
         The local potential vector.
     Returns
     -------
     int, float
         It returns the interface index in the locpot vector, and its `z` position
-        If the system does not contain an interface, it just returns the last spacial grid coordinate position
+        If the system does not contain an interface, it just returns the last spatial grid coordinate position
         and its corresponded index. -> [index,position] ==> [len(z)-1,z[-1]]
     '''
     peaks = find_peaks(x, y)
@@ -1827,10 +1866,10 @@ def plateau_val(x,y,critiria = 0.0001):
     Parameters
     ----------
     x : np.array
-        The spacial coordinates array.
+        The spatial coordinates array.
     y : np.array
-        The relevent functional values that corresponds the spacial coordinates vector.
-    critiria : float, optional, defualt: 0.0001
+        The relevent functional values that corresponds the spatial coordinates vector.
+    critiria : float, optional, default: 0.0001
         It can be changed for more of less strict evalutaion where the plateau has come.
     Returns
     -------
@@ -1867,7 +1906,7 @@ def is_peak_minimum(peaks, peak):
     Parameters
     ----------
     peaks : `numpt.ndarray`, (N,2)
-        The first column stands for the spacial poisition of the peaks.
+        The first column stands for the spatial poisition of the peaks.
         The second column is for the height of the peaks.
     peak : list: [int,float,float]
         The first argument is for the index of the peak inside the peaks array,
@@ -1896,8 +1935,8 @@ def is_peak_maximum(peaks, peak):
     Otherwise it will raise an index out-of-range error.
     Parameters
     ----------
-    peaks : `numpt.ndarray`, (N,2)
-        The first column stands for the spacial poisition of the peaks.
+    peaks : `numpy.ndarray`, (N,2)
+        The first column stands for the spatial poisition of the peaks.
         The second column is for the height of the peaks.
     peak : list: [int,float,float]
         The first argument is for the index of the peak inside the peaks array,
@@ -1953,4 +1992,84 @@ def is_peak_maximum_2(z,v, peak_pos):
     return flag
 
 def r_sqrd(v_original, v_fitted):
-    return 1 - (np.sum(((v_original - v_fitted) ** 2)) / np.sum((v_original - np.mean(v_original)) ** 2))
+    '''
+    Notes
+    -----
+    This function calculates the R square fitting parameter between two vectors.
+
+    Parameters
+    ----------
+    v_original : np.array
+        The vector contains the values whom we wish to compare to.
+    v_fitted : np.array
+        The vector contains the values whom we wish to fit .
+    Returns
+    -------
+    float
+        The calculated R sqrd value.
+    '''
+    if not len(to_1D_vec(v_original)) == len(to_1D_vec(v_fitted)):
+        return 0
+    else:
+        return 1 - (np.sum(((v_original - v_fitted) ** 2)) / np.sum((v_original - np.mean(v_original)) ** 2))
+
+def smooth_function(z,v):
+    '''
+    Notes
+    -----
+    This function calculates the R square fitting parameter between two vectors.
+
+    Parameters
+    ----------
+    z : np.array
+        The spatial coordinates array.
+    v : np.array
+        The relevent function's values that corresponds the spatial coordinates vector.
+    Returns
+    -------
+    np.array, np.array
+        This function returns the smoothed local potential vector with the corresponding spatial coordinates.
+    '''
+    flag_z = np.all(z == cons.m2A(z))
+    flag_v = np.all(v == cons.J2eV(v))
+    z = cons.m2A(z)
+    v = cons.J2eV(v)
+    window = len(v)
+    window = window*0.0000001
+    while np.fix(window) < 10:
+        window *=10
+    iteration = 0
+    while iteration < 100:
+        if np.mod(np.fix(window), 2) == 0:
+            try:
+                v_temp = savitzky_golay(v, window_size=np.fix(window) + 1, order=3)
+            except:
+                v_temp = np.ones(len(v))
+
+            if r_sqrd(v, v_temp) >= 0.96:
+                v = v_temp
+                break
+            else:
+                if iteration == 0:
+                    window = 1.0
+                else:
+                    window += 2
+        else:
+            try:
+                v_temp = savitzky_golay(v, window_size=np.fix(window), order=3)
+            except:
+                v_temp = np.ones(len(v))
+            if r_sqrd(v, v_temp) >= 0.96:
+                v = v_temp
+                break
+            else:
+                if iteration == 0:
+                    window = 1.0
+                else:
+                    window += 2
+        iteration +=1
+
+
+    z,v = interpolate_pchip(np.int64(len(z)*0.8),z,v)
+    z, v = interpolate_pchip(np.int64(len(z) * 1.25), z, v)
+    return z,v
\ No newline at end of file
